;;;; stack-eggdoc.scm

(use eggdoc)

(define license #<<EOF
Copyright (c) 2008, Mark Fredrickson.  All rights reserved.

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the Software),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED ASIS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
EOF
)

(define example1 #<<EOF
  (begin-stack
    9
    '(1 2 3 4 5)
    (push (lambda (x) (* x x)))
    map
    stk-swap
    (lambda (x) (lambda (y) (> y x))) ;; returns a function that will be pushed on the stack
    filter
    )
=> (stack 16 25)
EOF
)

(define example2 #<<EOF
  ;; words are easy to create by cut and paste!
  (define-word map-square-greater-filter
    (push (lambda (x) (* x x)))
    map
    stk-swap
    (lambda (x) (lambda (y) (> y x)))
    filter)

  (begin-stack
    9
    '(1 2 3 4 5)
    map-square-greater-filter)
=> (stack 16 25)
EOF
)

(define doc `(
	(eggdoc:begin
		(name "stacktor")
		(description (p "A stack based programming DSL, inspired by " (url "http://factorcode.org/" "Factor")))
		(author (url "http://chicken.wiki.br/mark-fredrickson" "Mark Fredrickson"))
		(history
			(version "0.8" "Release fix")
			(version "0.7" "Meta-file fix")
			(version "0.6" "Documentation fix")
			(version "0.5" "Initial release"))

		(usage)
		(download "stacktor.egg")

		(documentation
			(p "Stacktor allows programmers to use stack based programming next to more familiar styles of "
      "programming such as functional, imperative, object-oriented, etc. In stack programming, "
      "arguments are taken from the stack, and return values are pushed back on to the stack. Literal "
      "values are automatically pushed on to the stack. Stack operations (such as dropping the top "
      "element of the stack) allow programmers to shuffle order of arguments, duplicate arguments, and "
      "otherwise modify the stack as needed.")

      (p "Stacktor was inspired by " (url "http://factorcode.org/" "Factor") ", \"a practical stack language.\" Factor programs are composed of "
      "\"words\" and literals. Words are actions that occur on the stack. For example, the \"dup\" word "
      "takes the top of the stack and pushes an additional copy back on the stack. Words can also be "
      "composed of a series of other words, allowing the creation of richer levels of abstraction.")

      (p "In Stacktor you can define words using the (define-word ...) form; however, Stacktor will "
      "convert any Scheme procedure into a word automatically. Therefore, you can use existing Scheme "
      "procedures right inside of stack actions. When a procedure is found in a word or begin-stack "
      "form, Stacktor will pop the correct number of arguments (not including \"rest\" arguments) and "
      "apply the procedure to those arguments. The return value will be pushed onto the stack. Future "
      "versions of Stactor may push multiple return values onto the stack, though I haven't decided if "
      "that is a good idea yet.")

      (p "To begin using stack based programming in your code, use the (begin-stack ...) form. Each item "
      "in the form will be inspected at runtime to see if it is a literal value, a procedure, or a "
      "Stacktor \"word.\" This form will return a stack object, which you can convert to a list or "
      "process any other way you choose."
      )

			(group

				(macro "(begin-stack WORD ...)"
					(p "Returns a stack after operating with the supplied words. Words may be literal values (which are "
					   "pushed onto the stack), " (tt "word") " record types (typically created elsewhere), or procedures "
					   "(which are converted to words at runtime)."))

				(procedure "(push DATUM)"
					(p "Within the " (tt "begin-stack") " form, you can force pushing items onto the stack. This is most "
					   "useful when you want to push a procedure onto the stack, which would usually be converted to a word.")
					(pre "Example: (begin-stack '(1 2 3 4 5) (push (lambda (x) (* x x))) map) => (stack (1 4 9 16 25))")
					(p (i "This keyword may be come a keyword in future releases. Do not count on it remaining a procedure.")))
					
			  (macro "(begin-with-stack STACK WORD ...)"
			    (p "Identical to " (tt "begin-stack") " but also pasing in a stack, instead of creating a new one. Mostly "
			       "useful when defining an usual words, but even then consider other options."))
			    
			  (record "word"
			    (p "The word record is used to mark special procedures. Abstractly, word is a procedure with the signature of " 
			       (tt "STACK -> STACK") ". For convenience, these procedures are wrapped in a special record type. You "
			       "can certianly create words directly, but it is recommended you use the included word macros."))
			  (macro "(word WORD ...)"
			    (p "Takes one or more words and creates a " (tt "word") " record/procedure.")
			    (pre "Expands to: (make-word (lambda (stk) (begin-with-stack stk WORD ...)))"))
			    
			  (macro "(define-word NAME WORD ...)"
			    (p "Creates a new word record with name. Basically a convenice macro that in turn wraps " (tt "(word ...)"))
			    (pre "Expands to: (define NAME (word WORD ...))"))
			)
			
			(subsection "Predefined Words"
			  (p "By including the " (tt "stacktor-words") " unit, you can pick up some useful predfined words. The following are "
			      "taken from the " (url "http://factorcode.org/responder/help/show-help?topic=shuffle-words"  "Factor documentation") 
			      ". They use the Factor documentation style of defining the necessary input stack and the resulting stack. "
			      "All stack operations have been prefixed with " (tt "stk-") " to prevent name clashes.")
			  (symbol-table
			    (describe "stk-drop" "( x -- )")
          (describe "stk-2drop" "( x y -- )")
          (describe "stk-3drop" "( x y z -- )")
          (describe "stk-nip" "( x y -- y )")
          (describe "stk-2nip" "( x y z -- z )")
          (describe "stk-dup" "( x -- x x )")
          (describe "stk-2dup" "( x y -- x y x y )")
          (describe "stk-3dup" "( x y z -- x y z x y z )")
          (describe "stk-dupd" "( x y -- x x y )")
          (describe "stk-over" "( x y -- x y x )")
          (describe "stk-pick" "( x y z -- x y z x )")
          (describe "stk-tuck" "( x y -- y x y )")
          (describe "stk-swap" "( x y -- y x )")
          (describe "stk-swapd" "( x y z -- y x z )")
          (describe "stk-rot" "( x y z -- y z x )")
          (describe "stk--rot" "( x y z -- z x y )")
          (describe "stk-roll" "( x y z t -- y z t x )")
          (describe "stk--roll" "( x y z t -- t x y z )"))) ; end symbol table, subsection
          
		)
		(section "Examples"
		  (subsection "Find the squares of a list greater than a given value"
		    (pre ,example1))
		  (subsection "Turn the previous example into a word"
		    (pre ,example2)))
		(section "License" (pre ,license))
	)
))

(eggdoc->html doc)
